//
// ===== Requires =====
//
const Sequelize = require('sequelize');
const bcrypt    = require('bcryptjs');

//
// ===== Variable =====
//
var sequelize = new Sequelize({
    dialect: 'sqlite',
    storage: './credentials.db'
});
var User = sequelize.define('users', {
    username: {
        type: Sequelize.STRING,
        unique: true,
        allowNull: false
    },
    password: {
        type: Sequelize.STRING,
        allowNull: false
    },
    role: {
        type: Sequelize.STRING,
        allowNull: false
    }
});

//
// ===== Hooks =====
//
User.beforeCreate((user) => {
    const salt = bcrypt.genSaltSync();
    user.password = bcrypt.hashSync(user.password, salt);
});

// 
// ===== Methoden =====
//
User.prototype.validPassword = function(password) {
    return bcrypt.compareSync(password, this.password);
}

sequelize.sync()
    .then(() => console.log('users table has been successfully created, if one doesn\'t exist'))
    .catch(error => console.log('This error occured', error));

module.exports = User;
