//
// ===== Requires =====
//
const bodyParser    = require('body-parser');
const cookieParser  = require('cookie-parser');
const express       = require('express');
const http          = require('http');
const sio           = require('socket.io');
const session       = require('express-session');
const User          = require('./models/user');
const {
    AttributeIds,
    OPCUAClient,
    TimestampsToReturn,
} = require("node-opcua");
const endpointUrl_Rockwell01 = "opc.tcp://192.168.1.94:4990";


//
// ===== Variable =====
//
var app = express();
var server = http.createServer(app);
var io = sio(server);

// definiere nodeIDs ----------------
var nb_items = 12;
var nodeIDs = new Array(nb_items); 

nodeIDs[1]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]test_DINT",
	dataType: "Int32",
	htmlID: "htmlID_VP_data_01",
	browseName: "test_DINT",
};
nodeIDs[2]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]test_INT",
	dataType: "Int16",
	htmlID: "htmlID_VP_data_02",
	browseName: "test_INT",
};
nodeIDs[3]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]test_BOOL",
	dataType: "Boolean",
	htmlID: "htmlID_VP_data_03",
	browseName: "test_BOOL",
};
nodeIDs[4]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]test_STRING",
	dataType: "String",
	htmlID: "htmlID_VP_data_04",
	browseName: "test_STRING",
};
nodeIDs[5]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_station_name",
	dataType: "String",
	htmlID: "htmlID_VP_station_name",
	browseName: "VP: Station Name",
};
nodeIDs[6]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_name",
	dataType: "String",
	htmlID: "htmlID_VP_plc_name",
	browseName: "VP: PLC Name",
};
nodeIDs[7]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_ipAdd",
	dataType: "String",
	htmlID: "htmlID_VP_plc_ipAdd",
	browseName: "VP: PLC IP Address",
};
nodeIDs[8]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_errorText",
	dataType: "String",
	htmlID: "htmlID_VP_plc_errorText",
	browseName: "VP: PLC Error Text",
};
nodeIDs[9]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_alarmText",
	dataType: "String",
	htmlID: "htmlID_VP_plc_alarmText",
	browseName: "VP: PLC Alarm Text",
};
nodeIDs[10]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_communication",
	dataType: "String",
	htmlID: "htmlID_VP_plc_communication",
	browseName: "VP: PLC Communication",
};
nodeIDs[11]={
	endpointUrl: endpointUrl_Rockwell01,
	nodeID: "ns=3;s=[clx]opc_plc_machineStatus",
	dataType: "String",
	htmlID: "htmlID_VP_plc_machineStatus",
	browseName: "VP: PLC Machine Status",
};

var updateKeepalive = 0;

//
// ===== Settings =====
//
app.set('views', './views');
app.set('view engine', 'pug');
app.set('port', 3700);
app.use(bodyParser.urlencoded({extended: true}));
app.use(cookieParser());
app.use(session({
    key: 'user_sid',
    secret: 'somerandomsecretstuff',
    resave: false,
    cookie: {
        expires: 600000
    }
}));
app.use((req, res, next) => {
    if (req.cookies.user_sid && !req.session.user) {
        res.clearCookie('user_sid');
    }
    next();
});

server.listen(app.get('port'), () => console.log('App started'));

//
// ===== Routing =====
//
app.route('/')
    .get((req, res) => {
        if (req.session.user && req.cookies.user_sid) {
            res.redirect('/home');
        } else {
            res.redirect('/login');
        }
    });
app.route('/home')
    .get(async (req, res) => {
        if (req.session.user && req.cookies.user_sid) {
            var users = await User.findAll();
            res.render('home', {
                stations: [{
                    id: "HR",
                    name: "Station HR"
                }, {
                    id: "VP",
                    name: "Station VP",
                    members: [{
                        name: "HMI"
                    }, {
                        name: "Luftdruck"
                    }]
                }, {
                    id: "TS",
                    name: "Station TS"
                }, {
                    id: "MS",
                    name: "Station MS"
                }, {
                    id: "AS",
                    name: "Station AS"
                }],
                is_admin: req.session.user.role == "admin" ? true : false,
                users: users
            });
        } else {
            res.redirect('/login');
        }
        return false;
    });
app.route('/login')
    .get((req, res) => {
        if (req.session.user && req.cookies.user_sid) {
            res.redirect('/home');
        } else {
            res.render('login');
        }
    })
    .post((req, res) => {
        var username = req.body.username;
        var password = req.body.password;
        User.findOne({
            where: {
                username: username
            }
        }).then((user) => {
            if (!user) {
                res.render('login', {message: "Ungültiger Benutzer!"});
                console.log('Keinen User gefunden');
            } else if (!user.validPassword(password)) {
                res.render('login', {message: "Benutzer oder Passwort falsch!\nVersuche es noch einmal."});
                console.log('Passwort ist falsch!');
            } else {
                req.session.user = user.dataValues;
                res.redirect('/home');
                console.log('Login erfolgreich');
            }
        });
    });
app.route('/logout')
    .get((req, res) => {
        if (req.session.user && req.cookies.user_sid) {
            res.clearCookie('user_sid');
            res.redirect('/');
        } else {
            res.redirect('/login');
        }
    });
app.route('/register')
    .get((req, res) => {
        res.render('register');
    })
    .post((req, res) => {
        User.create({
            username: req.body.username,
            password: req.body.password,
            role: req.body.role
        }).then(user => {
            req.session.user = user.dataValues;
            res.redirect('/home');
        }).catch(error => {
            res.redirect('/register');
        });
    });

//
// ===== Fehlerbehandlung =====
//
app.use((req, res) => {
    res.status(404).send('Sorry, can\'t find that!');
});

(async () => {

    //
    // ===== OPC UA Client =====
    //
    // Start OPC UA Client
    const client = OPCUAClient.create({ endpoint_must_exist: false });
    client.on("backoff", (retry, delay) => { console.log("Retrying to connect to ", endpointUrl_Rockwell01, " attempt ", retry); });
    console.log(" connecting to " + endpointUrl_Rockwell01);
    await client.connect(endpointUrl_Rockwell01);
    console.log(" connected to " + endpointUrl_Rockwell01);

    const session = await client.createSession();
    console.log(" session created");

    const subscription = await session.createSubscription2({
    requestedPublishingInterval: 10,
    requestedMaxKeepAliveCount: 20,
    requestedLifetimeCount: 6000,
    maxNotificationsPerPublish: 1000,
    publishingEnabled: true,
    priority: 10
    });

    subscription.on("keepalive", function () {
    console.log("keepalive");
    updateKeepalive = 1;
    }).on("terminated", function () {
    console.log("TERMINATED")
    });

    // Node IDs to monitor    
    const parameters = {
    samplingInterval: 10,
    discardOldest: true,
    queueSize: 100
    };

    var itemsToMonitor = new Array(nb_items);
    var monitoredItems = new Array(nb_items);

    for (var i=1; i<nb_items; i++) {
        itemsToMonitor[i]= {
            nodeId: nodeIDs[i].nodeID,
            attributeId: AttributeIds.Value
        }
        monitoredItems[i] = await subscription.monitor(itemsToMonitor[i], parameters, TimestampsToReturn.Both);
    }

    // monitor the items and send to html client
    for (let i=1; i<nb_items; i++){
        monitoredItems[i].on("changed", (dataValue) => {
            io.sockets.emit('msg2c-1', {
            value: dataValue.value.value,
            timestamp: dataValue.serverTimestamp,
            nodeId: nodeIDs[i].nodeID,
            browseName: nodeIDs[i].browseName,
            htmlID: nodeIDs[i].htmlID,
            dataType: nodeIDs[i].dataType
            });
        });
    }

    // update Data on timer
    var updateTimer = setInterval(timerUpdateData, 1000);
    function timerUpdateData(){
        session.readVariableValue(nodeIDs[3].nodeID, function(err, dataValue) {
            if (!err) {
                //console.log('Timer update --> ' + nodeIDs[3].nodeID + ' --> ' + dataValue.value.value)
                io.sockets.emit('msg2c-1', {
                    value: dataValue.value.value,
                    timestamp: dataValue.serverTimestamp,
                    nodeId: nodeIDs[3].nodeID,
                    browseName: nodeIDs[3].browseName,
                    htmlID: nodeIDs[3].htmlID,
                    dataType: nodeIDs[3].dataType
                });
            }
        })

        session.readVariableValue(nodeIDs[10].nodeID, function(err, dataValue) {
            if (!err) {
                //console.log('Timer update --> ' + nodeIDs[10].nodeID + ' --> ' + dataValue.value.value)
                io.sockets.emit('msg2c-1', {
                    value: dataValue.value.value,
                    timestamp: dataValue.serverTimestamp,
                    nodeId: nodeIDs[10].nodeID,
                    browseName: nodeIDs[10].browseName,
                    htmlID: nodeIDs[10].htmlID,
                    dataType: nodeIDs[10].dataType
                });
            }
        })

        session.readVariableValue(nodeIDs[11].nodeID, function(err, dataValue) {
            if (!err) {
                //console.log('Timer update --> ' + nodeIDs[11].nodeID + ' --> ' + dataValue.value.value)
                io.sockets.emit('msg2c-1', {
                    value: dataValue.value.value,
                    timestamp: dataValue.serverTimestamp,
                    nodeId: nodeIDs[11].nodeID,
                    browseName: nodeIDs[11].browseName,
                    htmlID: nodeIDs[11].htmlID,
                    dataType: nodeIDs[11].dataType
                });
            }
        })
    }

    // detect CTRL+C and close
    var running = true;
    process.on("SIGINT", async () => {
        if (!running) {
            return; // avoid calling shutdown twice
        }
        console.log("shutting down client");
        running = false;

        await subscription.terminate();

        await session.close();
        await client.disconnect();
        console.log("Done");
        process.exit(0);
    });
})();

//
// ===== IO =====
//

(async () => {
    io.on('connection', (socket) => {
        socket.on('addUser', (info) => {
            console.log(info);
            User.create({
                username: info.username,
                password: info.password,
                role: info.role
            }).then(() => {
                socket.emit('addedUser', {username: info.username, role: info.role});
            });
        });
        socket.on('deleteUser', (info) => {
            console.log("delete User " + info.username);
            User.destroy({
                where: {
                    username: info.username
                }
            }).then(() => {
                socket.emit('deletedUser', {username: info.username});
            });
        });
        socket.on('msg2s-1', function (data) {
            console.log("---"+data.htmlID+"---"+data.value);
            var writeOK = 0;
            // find the correct nodeID				
            for(var i=1; i<nb_items; i++){
                if (data.htmlID === nodeIDs[i].htmlID){
                    var nodeIndex = i;
                }
            }
            if (data.htmlID === "htmlID_VP_data_01" || data.htmlID === "htmlID_VP_data_02" || data.htmlID === "htmlID_VP_data_03"){
                var nodeID_get = nodeIDs[nodeIndex].nodeID;
                var dataToWrite = {
                    dataType: nodeIDs[nodeIndex].dataType,
                    value: data.value
                }
                // check if the user entered the corract data type
                if (dataToWrite.dataType === "Int16") {
                    dataToWrite.value = parseInt(dataToWrite.value);
                    if (dataToWrite.value === parseInt(dataToWrite.value, 10) && dataToWrite.value >= -32768 && dataToWrite.value <= 32767){
                        writeOK = 1;
                    }
                }
                if (dataToWrite.dataType === "Int32") {
                    dataToWrite.value = parseInt(dataToWrite.value);
                    if (dataToWrite.value === parseInt(dataToWrite.value, 10) && dataToWrite.value >= -2147483648 && dataToWrite.value <= 2147483647){
                        writeOK = 1;
                    }
                }
                if (dataToWrite.dataType === "String") {
                    writeOK = 1;
                }
                if (dataToWrite.dataType === "Boolean") {
                    if (dataToWrite.value === "1" || dataToWrite.value === "true" || dataToWrite.value === "True"){
                        dataToWrite.value = true;
                        writeOK = 1;
                    }
                    if (dataToWrite.value === "0" || dataToWrite.value === "false" || dataToWrite.value === "False"){
                        dataToWrite.value = false;
                        writeOK = 1;
                    }
                }
                if (writeOK == 1){
                    session.writeSingleNode(nodeID_get, dataToWrite, function(err, statusCodes){
                        if(!err){
                            console.log("OK")
                        }else{
                            console.log("ERR")
                        }
                    });
                } else{
                    console.log("WARN: Wrong Data Type");
                }
            }
        });
    });
})();
